function res = ...
    pricingError(data, settings, parametersVect)
% This function computes the option pricing error (IVRMSE) between model and market implied
% volatility surfaces.
%% Check that parameters are within boundaries

global TechilaJobId
persistent iterationid
if isempty(iterationid)
    iterationid=0;
end

while true
    tic;
    if ~isempty(TechilaJobId)
        % Broadcast parameters from Job #1 to all other Jobs.
        parametersVect = cloudbc(parametersVect, 1);
        if parametersVect==-pi
            % All Jobs listen to the termination signal and complete the
            % Job when it is received.
            return
        end
        fprintf('after syncing parameters: %f\n', toc);
    end
    parameters.kappa = parametersVect(1);
    parameters.theta = parametersVect(2);
    parameters.xi = parametersVect(3);
    parameters.rho = parametersVect(4);
    parameters.V0 = parametersVect(5);
    parameters.gamma = parametersVect(6);
    
    inBounds = true;
    
    if parameters.kappa < settings.minKappa || parameters.kappa > settings.maxKappa || ...
            parameters.theta < settings.minTheta || parameters.theta > settings.maxTheta || ...
            parameters.xi < settings.minXi || parameters.xi > settings.maxXi || ...
            parameters.rho < settings.minRho || parameters.rho > settings.maxRho || ...
            parameters.gamma < settings.minGamma || parameters.gamma > settings.maxGamma || ...
            parameters.V0 < settings.minV0 || parameters.V0 > settings.maxV0
        inBounds = false;
    end
    
    if inBounds
        %% Filtering option contracts with respect to strike prices
        indK = data.K >= settings.StrikeRange(1) & data.K <= settings.StrikeRange(2);
        K = data.K(indK);
        T = data.T(indK);
        r = data.r(indK);
        S0 = data.S0(indK);
        IVol = data.IVol(indK);
        
        iterationid = iterationid + 1;
        
        %% Pricing options
        callPricesModel = MonteCarloPricing(S0, K, r, T, parameters, ...
            settings.nSim, settings.seedNumber, settings.tradingDaysInYear);
        fprintf('after MonteCarloPricing: %f\n', toc);
        if ~isempty(TechilaJobId)
            % Calculate the average model price using cloudsum.
            callPricesModel = cloudsum(callPricesModel) ./ getJobCount;
            fprintf('after cloudsum: %f\n', toc);
            if TechilaJobId > 1
                continue
            end
        end
        IVolModel = blsimpv(S0, K, r, T, callPricesModel);
        IVolModel(isnan(IVolModel)) = 0;
        fprintf('after postprocessing: %f\n', toc);
        
        
        
        %% Option pricing error
        res = 100*sqrt(mean((IVolModel - IVol).^2));
        
        %% Showing results
        if isempty(TechilaJobId)
            plotSurface(res, parameters, K, T, IVolModel, IVol, settings, 0);
        else
            % Techila Distributed Computing Engine functionality.
            % Saves the variables defined in the list into an intermediate data file,
            % which will be automatically transferred back to the End-User's computer.
            % The data in these variables will be used to visualize the progress.
            saveIMData('res','parameters','K','T','IVolModel','IVol', 'iterationid');
        end
        fprintf('after saveIMData: %f\n', toc);
        
    else
        res = 1e10;
    end
    
    if isempty(TechilaJobId) || TechilaJobId==1
        break
    end
    
end

end